@@ -1,8 +1,10 @@
 #' Utility function
 #' 
-#' @export
 #' @description Generate an empty plot window.
 #' 
+#' @export
+#' @import grDevices
+#' @import graphics
 #' @param xlim A one- or two-value vector indicating the range of the x-axis. 
 #' If \code{xlim} is a number, then it is assumed that the other value is 0. 
 #' Thus, \code{xlim=3000} wil result in a x-axis ranging from 0 to 3000, 
@@ -97,12 +99,17 @@ emptyPlot <- function(xlim, ylim,
 
 #' Utility function
 #' 
-#' @export
 #' @description Plot line with confidence intervals.
 #' 
+#' @export
+#' @import grDevices
+#' @import graphics
 #' @param x Vector with values on x-axis.
 #' @param fit Vector with values on y-axis.
-#' @param se.fit Vector with standard error.
+#' @param se.fit Vector with standard error; or when \code{se.fit2}
+#' is provided, \code{se.fit} specifies upper values confidence
+#' interval.
+#' @param se.fit2 Optional: lower values confidence interval.
 #' @param shade Logical: whether or not to produce shaded regions as 
 #' confidence bands.
 #' @param f Factor for converting standard error in confidence intervals. 
@@ -140,20 +147,38 @@ emptyPlot <- function(xlim, ylim,
 #' # Add arrows:
 #' abline(v=c(x$start, x$end), lty=3, col='red')
 #' arrows(x0=x$start, x1=x$end, y0=-5, y1=-5, code=3, length=.1, col='red')
-#' 
-#' # see the vignette on visualization, 
-#' # section 'Using predictions to customize plots', for an example:
-#' vignette(topic="plotfunctions")
+#'
+#' # Use of se.fit2:
+#' avg$cu <- avg$x[,'mean'] + avg$x[,'sd']
+#' avg$cl <- avg$x[,'mean'] - avg$x[,'sd']
+#' emptyPlot(range(avg$Time), c(-20,20), h0=0)
+#' plot_error(avg$Time, avg$x[,'mean'], se.fit=avg$cu, se.fit2=avg$cl, col='red')
+#' # Change layout:
+#' emptyPlot(range(avg$Time), c(-20,20), h0=0)
+#' plot_error(avg$Time, avg$x[,'mean'], avg$x[,'sd'], 
+#'    shade=TRUE, lty=3, lwd=3)
+#'
 #' @family Utility functions for plotting
 
-plot_error <- function(x, fit, se.fit, shade=FALSE, f=1, col='black', border=NA, alpha=.25, ...){
+plot_error <- function(x, fit, se.fit, se.fit2=NULL, 
+    shade=FALSE, f=1, col='black', border=NA, alpha=.25,  ...){
     if(shade){
         xval <- c(x, rev(x))
-        yval <- c(fit+f*se.fit, rev(fit-f*se.fit))
+        yval <- NA
+        if(is.null(se.fit2)){
+            yval <- c(fit+f*se.fit, rev(fit-f*se.fit))
+        }else{
+            yval <- c(se.fit, rev(se.fit2))
+        }
         polygon(x=xval, y=yval, border=border, col=alpha(col, f=alpha))
     }else{
-        lines(x, fit+f*se.fit, lty=2, col=col, ...)
-        lines(x, fit-f*se.fit, lty=2, col=col, ...)
+        if(is.null(se.fit2)){
+            lines(x, fit+f*se.fit, lty=2, col=col, ...)
+            lines(x, fit-f*se.fit, lty=2, col=col, ...)
+        }else{
+            lines(x, se.fit, lty=2, col=col, ...)
+            lines(x, se.fit2, lty=2, col=col, ...)
+        } 
     }
     lines(x, fit, col=col, ...)
 
@@ -163,6 +188,9 @@ plot_error <- function(x, fit, se.fit, shade=FALSE, f=1, col='black', border=NA,
 #' 
 #' @description Fill area under line or plot.
 #' 
+#' @export
+#' @import grDevices
+#' @import graphics
 #' @param x Vector with values on x-axis.
 #' @param y Vector with values on y-axis.
 #' @param from A number indicating until which value on the y-axis the graph 
@@ -216,6 +244,10 @@ fill_area <- function(x, y, from=0, col='black', alpha=.25,  border=NA, na.rm=TR
 #' Utility function
 #' 
 #' @export
+#' @export
+#' @import grDevices
+#' @import graphics
+#' @import stats
 #' @description Adjusted version of the a Cleveland dot plot implemented in 
 #' \code{\link[graphics]{dotchart}} with the option to add confidence 
 #' intervals.
@@ -346,6 +378,7 @@ dotplot_error <- function (x, se.val=NULL, labels = NULL, groups = NULL,
         groups <- groups[o]
         color <- rep_len(color, length(groups))[o]
         lcolor <- rep_len(lcolor, length(groups))[o]
+        bg <- rep_len(bg, length(groups))[o]
         offset <- cumsum(c(0, diff(as.numeric(groups)) != 0))
         y <- 1L:n + 2 * offset
         ylim <- range(0, y + 2)
@@ -382,3 +415,204 @@ dotplot_error <- function (x, se.val=NULL, labels = NULL, groups = NULL,
     title(main = main, xlab = xlab, ylab = ylab, ...)
     invisible()
 }
+
+
+#' Creates a contour plot with colored background.
+#'
+#' @description This function is a wrapper around \code{\link[graphics]{image}}
+#' and \code{\link[graphics]{contour}}. See \code{vignette("plotfunctions")} 
+#' for an example of how you could use \code{\link[graphics]{image}} and 
+#' \code{\link[graphics]{contour}}.
+#'
+#' @export
+#' @import grDevices
+#' @import graphics
+#' @param x Locations of grid lines at which the values in z are measured. 
+#' These must be in ascending order. By default, equally spaced values from 0 
+#' to 1 are used. If x is a list, its components x$x and x$y are used for x 
+#' and y, respectively. If the list has component z this is used for z.
+#' @param y Locations of grid lines at which the values in z are measured. 
+#' @param z a matrix containing the values to be plotted (NAs are allowed). 
+#' Note that x can be used instead of z for convenience.
+#' @param main Text string, an overall title for the plot.
+#' @param xlab Label for x axis. Default is name of first \code{view} variable.
+#' @param ylab Label for y axis. Default is name of second \code{view} 
+#' variable.
+#' @param xlim x-limits for the plot.
+#' @param ylim y-limits for the plot.
+#' @param zlim z-limits for the plot.
+#' @param col Color for the  contour lines and labels.
+#' @param color a list of colors such as that generated by 
+#' \code{\link[grDevices]{rainbow}}, \code{\link[grDevices]{heat.colors}}
+#' \code{\link[grDevices]{colors}}, \code{\link[grDevices]{topo.colors}}, 
+#' \code{\link[grDevices]{terrain.colors}} or similar functions.
+#' @param nCol The number of colors to use in color schemes.
+#' @param add.color.legend Logical: whether or not to add a color legend. 
+#' Default is TRUE. If FALSE (omitted), one could use the function
+#' \code{\link{gradientLegend}} to add a legend manually at any position.
+#' @param ... Optional parameters for \code{\link[graphics]{image}}
+#' and \code{\link[graphics]{contour}}.
+#' @author Jacolien van Rij
+#' @seealso \code{\link[graphics]{image}}, \code{\link[graphics]{contour}},
+#' \code{\link[graphics]{filled.contour}}. See \code{\link{plotsurface}}
+#' for plotting model predictions using \code{color_contour}.
+#' @examples
+#'
+#' # Volcano example of R (package datasets)
+#' color_contour(z=volcano)
+#' # change color and lines:
+#' color_contour(z=volcano, color='terrain', col=alpha(1), lwd=2, lty=5)
+#' # change x-axis values and zlim:
+#' color_contour(x=seq(500,700, length=nrow(volcano)),
+#'     z=volcano, color='terrain', col=alpha(1), lwd=2, zlim=c(0,200))
+#'
+#' \dontrun{
+#' # compare with similar functions:
+#' filled.contour(volcano, color.palette=terrain.colors)
+#' }
+#' # without contour lines:
+#' color_contour(z=volcano, color='terrain', lwd=0, drawlabels=FALSE)
+#' # without background:
+#' color_contour(z=volcano, color=NULL, add.color.legend=FALSE)
+#' @family Utility functions for plotting
+
+
+color_contour <- function(x = seq(0, 1, length.out = nrow(z)),
+    y = seq(0, 1, length.out = ncol(z)),
+    z,
+    main=NULL, xlab=NULL, ylab=NULL, 
+    xlim=NULL, ylim=NULL, zlim=NULL,
+    col=NULL, color=topo.colors(50), nCol=50,
+    add.color.legend=TRUE, ...){
+
+
+    # check input:
+    if(is.null(dim(z))){
+        stop('z should be a matrix.')
+    }
+    if(length(x) != nrow(z)){
+        stop(sprintf('x should have %d values, because z has %d rows.', nrow(z), nrow(z)))
+    }
+    if(length(y) != ncol(z)){
+        stop(sprintf('y should have %d values, because z has %d columns.', ncol(z), ncol(z)))
+    }
+        
+
+    ## Check plot settings
+    if(is.null(main)){
+        main=""
+    }
+    if(is.null(xlab)){
+        xlab=""
+    }
+    if(is.null(ylab)){
+        ylab=""
+    }
+    if(is.null(xlim)){
+        xlim=range(x)
+    }
+    if(is.null(ylim)){
+        ylim=range(y)
+    }   
+    if(is.null(zlim)){
+        zlim=range(z)
+    }   
+
+    # colors:
+    if(is.null(color)){
+        color <- alphaPalette('white', f.seq=c(0,0), n=nCol)
+    }
+    if (color[1] == "heat") {
+        color <- heat.colors(nCol)
+        if(is.null(col)){
+            col <- 3
+        }
+    } else if (color[1] == "topo") {
+        color <- topo.colors(nCol)
+        if(is.null(col)){
+            col <- 2
+        }
+    } else if (color[1] == "cm") {
+        color <- cm.colors(nCol)
+        if(is.null(col)){
+            col <- 1
+        }
+    } else if (color[1] == "terrain") {
+        color <- terrain.colors(nCol)
+        if(is.null(col)){
+            col <- 2
+        }
+    } else if (color[1] == "bpy") {
+        if (requireNamespace("sp", quietly = TRUE)) {
+            color <- sp::bpy.colors(nCol)
+            if(is.null(col)){
+                col <- 3
+            }
+        } else {
+            warning("Package 'sp' needed for bpy color palette. Using topo.colors instead (default).")
+            color <- topo.colors(nCol)
+            col <- 2
+        }
+    } else if (color[1] == "gray" || color[1] == "bw") {
+        color <- gray(seq(0.1, 0.9, length = nCol))
+        col <- 1
+    } 
+    if (is.null(col)){
+        col <- 'black'
+    } 
+
+
+    dnm <- list(...)
+    parlist <- names(dnm)
+
+    type2string <- function(x){
+        out <- ""
+        if(length(x)>1){
+            if(is.character(x)){
+                out <- sprintf("c(%s)", paste(sprintf("'%s'", x), collapse=','))
+            }else{
+                out <- sprintf("c(%s)", paste(x, collapse=','))
+            }
+        }else{
+            if(is.character(x)){
+                out <- sprintf("'%s'", x)
+            }else{
+                out <- sprintf("%s", x)
+            }
+        }
+        return(out)
+    }
+
+    # check contour input:
+    cpar <- c()
+    contourarg <- c('nlevels', 'levels', 'labels', 'labcex', 'drawlabels', 'method', 'lty', 'lwd')
+    for(i in parlist[parlist %in% contourarg] ){
+        cpar <- c(cpar, sprintf("%s=%s", i, type2string(dnm[[i]])))
+    }
+    cpar <- paste(",", paste(cpar, collapse=','))
+
+    cpar2 <- c()
+    for(i in parlist[parlist %in% c('nlevels', 'levels', 'method')] ){
+        cpar2 <- c(cpar2, sprintf("%s=%s", i, type2string(dnm[[i]])))
+    }
+    cpar2 <- paste(",", paste(cpar2, collapse=','))
+
+    # check image input:
+    ipar <- c()
+    contourarg <- c('nlevels', 'levels', 'labels', 'labcex', 'drawlabels', 'method', 'lty', 'lwd')
+
+    for(i in parlist[!parlist %in% contourarg] ){
+        ipar <- c(ipar, sprintf("%s=%s", i, type2string(dnm[[i]])))
+    }
+
+    ipar <- paste(",", paste(ipar, collapse=','))
+
+    eval(parse(text=sprintf("image(x, y, z, col=color, xlim=xlim, ylim=ylim, zlim=zlim, main=main, xlab=xlab, ylab=ylab, add=FALSE%s)", ipar)))
+    eval(parse(text=sprintf("contour(x, y, z, col=col, add=TRUE%s)",
+        cpar)))
+
+    if(add.color.legend){
+        gradientLegend(round(zlim, 3), n.seg=3, pos=.875, 
+            color=color)
+    }
+}
\ No newline at end of file
